@@ -53,6 +53,7 @@
 - (NSArray *)arrayWithFirstObjectRemoved:(NSArray *)array;
 - (void)removeItemWithDescription:(NSString *)desc;
 - (void)insertItem:(id)item;
+- (NSMutableDictionary *)flatten:(NSMutableDictionary *)dictSoFar;
 @end
 
 @implementation Directory
@@ -216,6 +217,12 @@
 	return [dir addNewDirectory:[self arrayWithFirstObjectRemoved:pathArray]];
 }
 
+// adds to this directory - see addItem:toDir:
+- (Directory *)addItem:(MyItem *)file
+{
+	return [self addItem:file toDir:[NSArray array]];
+}
+
 // returns the directory the item was added to, and creates it if necessary
 - (Directory *)addItem:(MyItem *)item toDir:(NSArray *)pathArray
 {
@@ -297,31 +304,91 @@
 	return [contents itemWithDescription:desc];
 }
 
-/* builds up recursively an array of all items here and below.
- * Includes directories but not the initial receiver directory
- * (the others get added when this message is sent to their
- * containing directory.  The deepest down items are returned
- * first.
- */
-- (NSMutableArray *)recursiveMembers
-{
-	NSMutableArray *items = [[NSMutableArray alloc] init];
-	NSEnumerator *enumerator = [contents objectEnumerator];
-	id obj;
-	while (obj = [enumerator nextObject])
-	{
-		if ([obj isMemberOfClass:[Directory class]])
-			[items addObjectsFromArray:[obj recursiveMembers]];
-		[items addObject:obj];
-	}	
-	return [items autorelease];
-}
-
 - (Directory *)copy
 {
-	Directory *copy = [[Directory alloc] initWithName:name withContents:contents];
+//	Directory *copy = [[Directory alloc] initWithName:name withContents:contents];
 	
-	return [copy autorelease];
+	// pretending to be immutable for now...
+	return [self retain];
+	
+//	return [copy autorelease];
+}
+
+- (Directory *)subdirWithID:(unsigned)dirID
+{
+	if ([self itemID] == dirID)
+		return self;
+	
+	NSEnumerator *enumerator = [self objectEnumerator];
+	MyItem *currentItem;
+	while (currentItem = [enumerator nextObject])
+	{
+		if ([currentItem isMemberOfClass:[Directory class]])
+		{
+			Directory *dir = [currentItem subdirWithID:dirID];
+			if (dir != nil)
+				return dir;
+		}
+	}
+	return nil;
+}
+
+- (NSDictionary *)flatten
+{
+	NSMutableDictionary *dict = [self flatten:[[[NSMutableDictionary alloc] init] autorelease]];
+	// make unmutable
+  return [NSDictionary dictionaryWithDictionary:dict];
+}
+
+- (NSMutableDictionary *)flatten:(NSMutableDictionary *)dictSoFar
+{
+	[dictSoFar setObject:self forKey:[NSNumber numberWithUnsignedInt:[self itemID]]];
+	
+	NSEnumerator *enumerator = [self objectEnumerator];
+	MyItem *currentItem;
+	while (currentItem = [enumerator nextObject])
+	{
+		if ([currentItem isMemberOfClass:[Directory class]])
+		{
+			[currentItem flatten:dictSoFar];
+		}
+	}
+	return dictSoFar;
+}
+
+// removes empty entries at the beginning/end of the array
+// that come from leading/trailing separators on paths from componentsSeparatedByString
++ (NSArray *)normalizePathArray:(NSArray *)pathArray
+{
+	if ([pathArray count] == 0)
+		return pathArray;
+	
+	NSMutableArray *newPathArray = [NSMutableArray arrayWithArray:pathArray];
+	if ([[newPathArray objectAtIndex:[newPathArray count] - 1] length] == 0)
+		[newPathArray removeLastObject];
+	
+	if ([pathArray count] > 0)
+		if ([[newPathArray objectAtIndex:0] length] == 0)
+			[newPathArray removeObjectAtIndex:0];
+	
+	return [NSArray arrayWithArray:newPathArray];
+}
+
+// returns a path string with '\' as separators with leading and trailing '\'
++ (NSString *)stringFromPathArray:(NSArray *)pathArray
+{
+	NSMutableString *path = [[NSMutableString alloc] init];
+	NSEnumerator *enumerator = [pathArray objectEnumerator];
+	NSString *pathItem;
+	while (pathItem = [enumerator nextObject])
+	{
+		if ([pathItem length] > 0)
+			[path appendFormat:@"\\%@", pathItem];
+	}
+	[path appendString:@"\\"];
+	NSString *pathImmutable = [NSString stringWithString:path];
+	[path release];
+	return pathImmutable;
 }
 
 @end
