@@ -1766,13 +1766,20 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 	else
 		libmtp_track->artist = strdup([[track artist] UTF8String]);
 	
-	libmtp_track->genre = strdup([[track genre] UTF8String]);
+	if ([[track genre] length] > 0)
+		libmtp_track->genre = strdup([[track genre] UTF8String]);
+	else
+		libmtp_track->genre = strdup("");
+	
 	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T000000.0", [track year]];
 	libmtp_track->date = strdup([nsTimeString UTF8String]);
 	libmtp_track->tracknumber = [track trackNumber];
 	libmtp_track->duration = [track length] * 1000;
 	libmtp_track->filesize = [track filesize];
-	libmtp_track->filename = strdup([[track filename] UTF8String]);
+	if ([[track filename] length] > 0)
+		libmtp_track->filename = strdup([[track filename] UTF8String]);
+	else
+		libmtp_track->filename = NULL;
 	libmtp_track->item_id = [track itemID];
 		
 	return libmtp_track;
